class Parameter < ActiveRecord::Base
  include Trimmer
  trimmed_fields :key

  belongs_to :parameterable, :polymorphic => true
  validates_presence_of :key

  # attr_readonly :created_at # FIXME: these should be readonly, but inherit_resources isn't creating new instances right
  attr_accessible :created_at # FIXME: ^^
  attr_accessible :key, :value, :parameterable_id, :parameterable_type, :updated_at

  before_validation :parse_value

  serialize :value

  fires :added_to,      :on => :create,   :secondary_subject => 'parameterable'
  fires :removed_from,  :on => :destroy,  :secondary_subject => 'parameterable'
  fires :changed_on,    :on => :update,   :secondary_subject => 'parameterable'

  def name
    if (self.parameterable_type == 'Node' || self.parameterable_type == 'NodeGroup')
      return "Variable '#{key}'"
    else
      return "Parameter '#{key}'"
    end
  end

  # Converts the value to a boolean if it represents one. Otherwise, the
  # original value is returned. For the most part, Puppet doesn't convert these
  # strings to their boolean values, so we have to do it here instead, lest
  # 'false' be interpreted as true and we all succumb to madness.
  #
  # It would be nice to do this with numbers as well, but since parsing a
  # number and then stringifying it can result in a different string, there's a
  # risk of breaking when a string simply *looks* like a number but isn't meant
  # to represent one (such as a -0 suffix, etc).
  def parse_value
    self.value = case self.value
                 when 'true'
                   true
                 when 'false'
                   false
                 else
                   self.value
                 end

    self
  end
end
